/*
 * Rubber Spoon libSteg 0.4.3
 * by Joel R. Voss
 * August 16, 2004 10:55
 * SEE rs04.txt for more details.
 */

#include "rs043.h"
#include <string.h>
#include <stdio.h>

#ifdef USE_SSL
	#include <openssl/sha.h>
	#define SHA1_wrap SHA1
#else
	// Use Libtomcrypt wrapper
	#include "sha_wrap1.h"
	#define SHA1_wrap SHA1_libtomcrypt
#endif

// 1a) data bits -> data byte
// convertBitToByte converts one bit to a byte with the bit in the lowest 
// position. if bit is true, 1, else 0.
BYTE convertBitToByte(BYTE input, BYTE bit)
{
	// Bit must be between 0-7, else throw error.
	if(bit > 7) return 0;
	BYTE mask = 0x1 << bit;
	//printf("mask = %x;\n", mask);
	return (input & mask) >> bit;
}

// 1c) data bits -> data byte
// convertDataToBinary converts a N bytes to 8 * K * N bytes with each bit in 
// the lowest position of the byte it created.
void convertDataToKeyBinary(BYTE *data, DWORD size, int *key, DWORD key_size, BYTE *dest)
{
	// pos = -key[key_size-1] is a workaround hack to start at zero.
	int index, pos = -key[key_size-1]-1;
	
	for(index = 0; index < size; index++)
	{
		int bit;
		for(bit = 0; bit < 8; bit++)
		{
			// Loop through the key using this function.
			if((index * 8 + bit) % key_size == 0) pos += key[key_size-1]+1;
			dest[key[(index * 8 + bit) % key_size] + pos] = convertBitToByte(data[index], 7-bit);
		}
	}
}

// The need to key stegs has created the following function.
void StegKey(char *SecretCode, int *KeyHash)
{
	//turn secret code into a sha1 hash
	static unsigned char HashOutput[SHA_DIGEST_LENGTH+1];
	
	// FIXME: To be resistant against rainbow tables, we need to add a salt.
	
	/* users secret word, must be null terminated. */
	SHA1_wrap(SecretCode, strlen(SecretCode), HashOutput);
	
	// This is a hack to ensure that we start at position zero.
	// See the piece = 1 + ...
	int PrevPos = -1;
	int i, bpos, pos, piece;
	
	for(i = 0; i < 50; i++)
	{
		// bpos is which byte of the has output we're looking at.
		bpos = (3 * i) / 8;
		// pos is the bit in that byte that we're looking at.
		pos  = (3 * i) % 8;
		// 0x7 = 00000111 binary.
		// see ../research/shamain3.c for explanation of short and 1+...
		piece = 1 + (((*(short *)&(HashOutput[bpos])) & (0x7 << pos)) >> pos);
		KeyHash[i] = piece + PrevPos;
		PrevPos = KeyHash[i];
	}
	//output is KeyHash
}

// To do no key, do not call StegKey.
// Instead use the following function.
void StegNoKey(int *KeyHash)
{
	int i;
	for(i = 0; i < 50; i++)
	{
		KeyHash[i] = i + 1;
	}
}


// The KeyMask puts ff where there is no key, fe where there is a key.
// data AND fe deletes the least significant bit of data.
void createKeyMask(int *key, DWORD key_size, BYTE *dest)
{
	// Inverse Mask = 0xfe = 254 = 11111110
	int index;
	
	// All are initialized ff which means no.
	for(index = 0; index < key[key_size-1]; index++)
	{
		dest[index] = 0xff;
	}
	
	// The ones that are keyed need to be fe 
	// to delete the least significant bit.
	for(index = 0; index < key_size; index++)
	{
		dest[key[index]] = 0xfe;
	}
}

// 3a) dest1 = pixels and inverse mask
// andData ands all the data in a and b.
void andData(BYTE *a, DWORD asize, BYTE *b, DWORD bsize, BYTE *dest, DWORD size)
{
	int index;
	
	for(index = 0; index < size; index++)
	{
		dest[index] = a[index % asize] & b[index % bsize];
	}
}

// 3b) dest2 = dest1 or data bytes
// orData ors all the data in a and b.
void orData(BYTE *a, DWORD asize, BYTE *b, DWORD bsize, BYTE *dest, DWORD size)
{
	int index;
	
	for(index = 0; index < size; index++)
	{
		dest[index] = a[index % asize] | b[index % asize];
	}
}

// steg: A simple function to take data and interlace it with pixels of an 
// image such that an onlooker will not see the data. <1>
void steg(BYTE *data, DWORD size, int *key, DWORD key_size, BYTE *pixels, BYTE *output)
{
	// bytes = size * 8 bits * key[key_size - 1] per 50 bits
	int real_size = size * 8 * (key[key_size - 1]+1) / 50;
	
	// The Keyed LSB Data.
	BYTE    bytes[real_size];
	memset(bytes, 0, real_size);
	
	BYTE  keymask[key[key_size - 1]];
	
	// Key Mask is designed to delete the least significant bit in a key 
	// pattern when you call andData(pixels, keymask).
	createKeyMask(key, key_size, keymask);
	
	// 1) data bits -> keyed data bytes
	convertDataToKeyBinary(data, size, key, key_size, bytes);
	
	// And Data with keymask deletes the least significant bit.
	andData(pixels, real_size, keymask, key[key_size - 1]+1, output, real_size);
	
	// Or Data with lsb data puts the data in the least significant bit.
	orData(output, real_size, bytes, real_size, output, real_size);

}

#ifdef TEST

#include <stdio.h>

// Test libSteg steg 0.4
int main(int argc, char **argv)
{
	BYTE data[] = {231, 14, 32, 16, 77};
	int size = 5;
	
	// SHA Hash Key
	int key[50];
	
	StegKey("javantea", key);
	
	int real_size = size * 8 * key[49] / 50;
	printf("rs=%i\n", real_size);
	
	BYTE   pixels[real_size];
	BYTE   output[real_size];
	
	// Get a suitable image.
	fread(pixels, real_size, 1, stdin);

	// The actual stegging.
	steg(data, size, key, 50, pixels, output);
	
	// Produce a copy of the image that contains the stegged data.
	fputs("P5\n39 4\n255\n", stdout);
	fwrite(output, real_size, 1, stdout);
	printf("\n");
	return 0;
}
#endif // TEST

/* <2> For example, if I had a stegged picture with a good key, a person could 
 * brute force the key and get several messages: one might say "meet me at 6" 
 * another might say, "terror at 7 pm", while another might say "i R 1337". 
 * Which is the most likely? Probably the third.
 */
