/*
 * Rubber Spoon libSteg 0.4
 * DeSteg
 * by Joel R. Voss
 * August 16, 2004 10:55
 * SEE rs04.txt and rs043.c for more details.
 */

#include "rs043r.h"
#include <string.h>

// 4a) data byte -> data bits
// convertByteToBit converts one byte to a bit using the bit in the lowest 
// position. if the lowest bit is true, 1 << bit, else 0.
BYTE convertByteToBit(BYTE input, BYTE bit)
{
	// Bit must be between 0-7, else throw error.
	if(bit > 7) return 0;
	// Take the least significant bit, shift it to the left into it's 
	// correct position.
	return (input & 0x1) << bit;
}

// 4c) data bits -> data ints
// convertDataToBinary converts a N bytes to 8 * N bytes with each bit in the 
// lowest position of the byte it created.
void convertKeyBinaryToData(BYTE *data, DWORD size, int *key, DWORD key_size, BYTE *pixels)
{
	// pos = -key[key_size-1] is a workaround hack to start at zero.
	int index, bit, pos = -key[key_size-1]-1;
	
	for(index = 0; index < size; index++)
	{
		// In order for the OR below to work, the data must be 0.
		data[index] = 0;
		for(bit = 0; bit < 8; bit++)
		{
			// Loop through the key using this function.
			if((index * 8 + bit) % key_size == 0) pos += key[key_size-1]+1;
			// So each bit is put into this byte by OR.
			data[index] |= convertByteToBit(pixels[key[(index * 8 + bit) % key_size] + pos], 7-bit);
		}
		
	}
}


// desteg: A simple function to reverse the steg effect on pixels of an 
// image such that an onlooker will not see the data. <1>
void desteg(BYTE *data, DWORD size, int *key, DWORD key_size, BYTE *pixels)
{
	// 1) data bits -> data bytes
	convertKeyBinaryToData(data, size, key, key_size, pixels);
}

#ifdef TEST

// Need to include rs043.h because it has StegKey()
#include "rs043.h"
#include <stdio.h>

// Test libSteg desteg 0.4
int main(int argc, char **argv)
{
	int size = 5;
	BYTE data[size];

	// SHA Hash Key
	int key[50];
	
	StegKey("javantea", key);
	
	int real_size = size * 8 * key[49] / 50;
	printf("rs=%i\n", real_size);
	
	BYTE   pixels[real_size];
	
	// Get the stegged image.
	fread(pixels, real_size, 1, stdin);
	
	// The actual destegging.
	desteg(data, size, key, 50, pixels);
	
	// Display the data
	printf("data: ");
	int index;
	for(index = 0; index < size; index++)
	{
		printf("%i ", data[index]);
	}
	printf("\n");
	for(index = 0; index < size; index++)
	{
		printf("%c", data[index]);
	}
	printf("\n");
	return 0;
}
#endif // TEST
